@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -217,6 +217,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
             except httplib.BadStatusLine:
                 # some servers send empty HEAD replies
                 if self.method == "HEAD":
+                    log.debug(LOG_CHECK, "Empty status line: falling back to GET")
                     self.method = "GET"
                     self.aliases = []
                     self.fallback_get = True
@@ -249,6 +250,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
             except httplib.BadStatusLine:
                 # some servers send empty HEAD replies
                 if self.method == "HEAD":
+                    log.debug(LOG_CHECK, "Empty status line: falling back to GET")
                     self.method = "GET"
                     self.aliases = []
                     self.fallback_get = True
@@ -311,8 +313,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
             newurl = self.headers.getheader("Location",
                          self.headers.getheader("Uri", ""))
             # make new url absolute and unicode
-            newurl = urlparse.urljoin(redirected, newurl)
             newurl = unicode_safe(newurl)
+            newurl = urlparse.urljoin(redirected, newurl)
             log.debug(LOG_CHECK, "Redirected to %r", newurl)
             self.add_info(_("Redirected to `%(url)s'.") % {'url': newurl})
             # norm base url - can raise UnicodeError from url.idna_encode()
@@ -325,6 +327,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
             self.set_extern(redirected)
             if self.extern[0] and self.extern[0]:
                 if set_result:
+                    self.check301status(response)
                     self.add_info(
                           _("The redirected URL is outside of the domain "
                             "filter, checked only syntax."))
@@ -357,14 +360,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 self.aliases.append(redirected)
             # note: urlparts has to be a list
             self.urlparts = urlparts
-            if response.status == 301:
-                if not self.has301status:
-                    if set_result:
-                        self.add_warning(
-                           _("HTTP 301 (moved permanent) encountered: you"
-                             " should update this link."),
-                           tag=WARN_HTTP_MOVED_PERMANENT)
-                    self.has301status = True
+            if set_result:
+                self.check301status(response)
             # check cache again on the changed URL
             if self.aggregate.urlqueue.checked_redirect(redirected, self):
                 return -1, response
@@ -390,6 +387,14 @@ Use URL `%(newurl)s' instead for checking.""") % {
             tries += 1
         return tries, response
 
+    def check301status (self, response):
+        """If response page has been permanently moved add a warning."""
+        if response.status == 301 and not self.has301status:
+            self.add_warning(_("HTTP 301 (moved permanent) encountered: you"
+                               " should update this link."),
+                             tag=WARN_HTTP_MOVED_PERMANENT)
+            self.has301status = True
+
     def get_alias_cache_data (self):
         """
         Return all data values that should be put in the cache,
@@ -445,8 +450,15 @@ Use URL `%(newurl)s' instead for checking.""") % {
         except socket.error, msg:
             if msg.args[0] == 32 and self.reused_connection:
                 # server closed persistent connection - retry
+                log.debug(LOG_CHECK, "Server closed connection: retry")
+                self.persistent = False
+                return self._get_http_response()
+            raise
+        except httplib.BadStatusLine, msg:
+            if str(msg) == "Empty status line" and self.reused_connection:
+                # server closed connection - retry
+                log.debug(LOG_CHECK, "Empty status line: retry")
                 self.persistent = False
-                self.close_connection()
                 return self._get_http_response()
             raise
 
@@ -456,7 +468,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
         """
         if self.proxy:
             host = self.proxy
-            scheme = "http"
+            scheme = self.proxytype
         else:
             host = self.urlparts[1]
             scheme = self.urlparts[0]
@@ -659,6 +671,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
             self.parse_css()
         elif ctype == "application/x-shockwave-flash":
             self.parse_swf()
+        elif ctype == "application/msword":
+            self.parse_word()
 
     def get_robots_txt_url (self):
         """
