@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,9 +11,9 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Base URL handler.
 """
@@ -27,15 +27,15 @@ import time
 import errno
 import socket
 import select
-import traceback
+import tempfile
 
 from . import absolute_url, StoringHandler, get_url_from
 from ..cache import geoip
 from .. import (log, LOG_CHECK, LOG_CACHE, httputil, httplib2 as httplib,
-    strformat, containers, LinkCheckerError, url as urlutil,
-    trace, clamav)
+    strformat, LinkCheckerError, url as urlutil, trace, clamav, containers,
+    winutil)
 from ..HtmlParser import htmlsax
-from ..htmlutil import linkparse, titleparse
+from ..htmlutil import linkparse
 from .const import (WARN_URL_EFFECTIVE_URL, WARN_URL_UNICODE_DOMAIN,
     WARN_URL_UNNORMED, WARN_URL_ERROR_GETTING_CONTENT,
     WARN_URL_ANCHOR_NOT_FOUND, WARN_URL_WARNREGEX_FOUND,
@@ -55,12 +55,11 @@ def urljoin (parent, url, scheme):
     return urlparse.urljoin(parent, url)
 
 
-def url_norm (url):
-    """
-    Wrapper for url.url_norm() to convert UnicodeError in LinkCheckerError.
-    """
+def url_norm (url, encoding=None):
+    """Wrapper for url.url_norm() to convert UnicodeError in
+    LinkCheckerError."""
     try:
-        return urlutil.url_norm(url)
+        return urlutil.url_norm(url, encoding=encoding)
     except UnicodeError:
         msg = _("URL has unparsable domain name: %(name)s") % \
             {"name": sys.exc_info()[1]}
@@ -71,8 +70,8 @@ class UrlBase (object):
     """An URL with additional information like validity etc."""
 
     def __init__ (self, base_url, recursion_level, aggregate,
-                  parent_url = None, base_ref = None,
-                  line = -1, column = -1, name = u""):
+                  parent_url=None, base_ref=None, line=-1, column=-1,
+                  name=u"", url_encoding=None):
         """
         Initialize check data, and store given variables.
 
@@ -84,15 +83,16 @@ class UrlBase (object):
         @param line: line number of url in parent content
         @param column: column number of url in parent content
         @param name: name of url or empty
+        @param url_encoding: encoding of URL or None
         """
         self.init(base_ref, base_url, parent_url, recursion_level,
-                  aggregate, line, column, name)
+                  aggregate, line, column, name, url_encoding)
         self.reset()
         self.check_syntax()
 
 
     def init (self, base_ref, base_url, parent_url, recursion_level,
-              aggregate, line, column, name):
+              aggregate, line, column, name, url_encoding):
         """
         Initialize internal data.
         """
@@ -105,6 +105,7 @@ class UrlBase (object):
         self.line = line
         self.column = column
         self.name = name
+        self.encoding = url_encoding
         if self.base_ref:
             assert not urlutil.url_needs_quoting(self.base_ref), \
                    "unquoted base reference URL %r" % self.base_ref
@@ -130,6 +131,8 @@ class UrlBase (object):
         self.urlparts = None
         # the anchor part of url
         self.anchor = None
+        # list of parsed anchors
+        self.anchors = []
         # the result message string and flag
         self.result = u""
         self.has_result = False
@@ -138,9 +141,9 @@ class UrlBase (object):
         # valid or not
         self.valid = True
         # list of warnings (without duplicates)
-        self.warnings = containers.SetList()
-        # list of infos (without duplicates)
-        self.info = containers.SetList()
+        self.warnings = []
+        # list of infos
+        self.info = []
         # download time
         self.dltime = -1
         # download size
@@ -181,7 +184,11 @@ class UrlBase (object):
         """Return title of page the URL refers to.
         This is per default the filename or the URL."""
         if self.title is None:
-            url = self.url if self.url else self.base_url
+            url = u""
+            if self.base_url:
+                url = self.base_url
+            elif self.url:
+                url = self.url
             self.title = url
             if "/" in url:
                 title = url.rsplit("/", 1)[1]
@@ -193,14 +200,20 @@ class UrlBase (object):
         """Set title of page the URL refers to.from page content."""
         if self.valid and self.is_html():
             try:
-                handler = titleparse.TitleFinder(self.get_content())
+                handler = linkparse.TitleFinder(self.get_content())
             except tuple(ExcList):
                 return
             parser = htmlsax.parser(handler)
             handler.parser = parser
             # parse
-            parser.feed(self.get_content())
-            parser.flush()
+            try:
+                parser.feed(self.get_content())
+                parser.flush()
+            except linkparse.StopParse, msg:
+                log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
+            # break cyclic dependencies
+            handler.parser = None
+            parser.handler = None
             if handler.title:
                 self.title = handler.title
 
@@ -236,13 +249,16 @@ class UrlBase (object):
         """
         Add a warning string.
         """
-        self.warnings.append((tag, s))
+        item = (tag, s)
+        if item not in self.warnings:
+            self.warnings.append(item)
 
-    def add_info (self, s, tag=None):
+    def add_info (self, s):
         """
         Add an info string.
         """
-        self.info.append((tag, s))
+        if s not in self.info:
+            self.info.append(s)
 
     def copy_from_cache (self, cache_data):
         """
@@ -250,28 +266,35 @@ class UrlBase (object):
         """
         self.result = cache_data["result"]
         self.has_result = True
-        self.warnings.extend(cache_data["warnings"])
-        self.info.extend(cache_data["info"])
+        for tag, msg in cache_data["warnings"]:
+            # do not copy anchor warnings, since the current anchor
+            # might have changed
+            if tag != WARN_URL_ANCHOR_NOT_FOUND:
+                self.add_warning(msg, tag=tag)
+        for info in cache_data["info"]:
+            self.add_info(info)
         self.valid = cache_data["valid"]
         self.dltime = cache_data["dltime"]
         self.dlsize = cache_data["dlsize"]
+        self.anchors = cache_data["anchors"]
         self.cached = True
+        # recheck anchor
+        if self.valid and self.anchor:
+            self.check_anchor()
 
     def get_cache_data (self):
-        """
-        Return all data values that should be put in the cache.
-        """
+        """Return all data values that should be put in the cache."""
         return {"result": self.result,
                 "warnings": self.warnings,
                 "info": self.info,
                 "valid": self.valid,
                 "dltime": self.dltime,
                 "dlsize": self.dlsize,
+                "anchors": self.anchors,
                }
 
     def get_alias_cache_data (self):
-        """
-        Return all data values that should be put in the cache.
+        """Return all data values that should be put in the cache.
         Intended to be overridden by subclasses that handle aliases.
         """
         return self.get_cache_data()
@@ -286,15 +309,7 @@ class UrlBase (object):
         assert isinstance(self.cache_content_key, unicode), self
         log.debug(LOG_CACHE, "Content cache key %r", self.cache_content_key)
         # construct cache key
-        if self.aggregate.config["anchorcaching"] and \
-           self.aggregate.config["anchors"]:
-            # do not ignore anchor
-            parts = self.urlparts[:]
-            parts[4] = self.anchor
-            self.cache_url_key = urlparse.urlunsplit(parts)
-        else:
-            # no anchor caching
-            self.cache_url_key = self.cache_content_key
+        self.cache_url_key = self.cache_content_key
         assert isinstance(self.cache_url_key, unicode), self
         log.debug(LOG_CACHE, "URL cache key %r", self.cache_url_key)
 
@@ -304,9 +319,6 @@ class UrlBase (object):
         url syntax. Success enables further checking, failure
         immediately logs this url. Syntax checks must not
         use any network resources.
-
-        @return: True if syntax is correct, else False.
-        @rtype: bool
         """
         log.debug(LOG_CHECK, "checking syntax")
         if self.base_url is None:
@@ -335,7 +347,7 @@ class UrlBase (object):
         url information self.base_url, self.parent_url and self.base_ref.
         """
         # norm base url - can raise UnicodeError from url.idna_encode()
-        base_url, is_idn = url_norm(self.base_url)
+        base_url, is_idn = url_norm(self.base_url, self.encoding)
         if is_idn:
             self.add_warning(_("""URL %(url)r has a unicode domain name which
                           is not yet widely supported. You should use
@@ -429,20 +441,16 @@ class UrlBase (object):
         try:
             self.check_connection()
             self.add_country_info()
-            if self.aggregate.config["anchors"]:
-                self.check_anchors()
+            self.check_content()
         except tuple(ExcList):
             value = self.handle_exception()
             # make nicer error msg for unknown hosts
-            if isinstance(value, socket.error) and value[0] == -2:
+            if isinstance(value, socket.error) and value.args[0] == -2:
                 value = _('Hostname not found')
             # make nicer error msg for bad status line
             if isinstance(value, httplib.BadStatusLine):
                 value = _('Bad HTTP response %(line)r') % {"line": str(value)}
             self.set_result(unicode_safe(value), valid=False)
-        if self.can_get_content():
-            self.set_title_from_content()
-            self.check_content()
         self.checktime = time.time() - check_start
         # check recursion
         try:
@@ -473,11 +481,11 @@ class UrlBase (object):
         """
         An exception occurred. Log it and set the cache flag.
         """
-        etype, value, tb = sys.exc_info()
-        log.debug(LOG_CHECK, "exception %s", traceback.format_tb(tb))
+        etype, value = sys.exc_info()[:2]
+        log.debug(LOG_CHECK, "Error in %s: %s %s", self.url, etype, value, exception=True)
         # note: etype must be the exact class, not a subclass
         if (etype in ExcNoCacheList) or \
-           (etype == socket.error and value[0]==errno.EBADF) or \
+           (etype == socket.error and value.args[0]==errno.EBADF) or \
             not value:
             # EBADF occurs when operating on an already socket
             self.caching = False
@@ -537,37 +545,52 @@ class UrlBase (object):
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
-        parser.feed(self.get_content())
-        parser.flush()
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
         parser.handler = None
         return handler.follow
 
-    def check_anchors (self):
-        """
-        If URL was valid and a HTML resource, check the anchors and
-        log a warning when an anchor was not found.
-        """
-        if not (self.valid and self.anchor and self.is_html() and \
-                self.can_get_content()):
-            # do not bother
-            return
-        log.debug(LOG_CHECK, "checking anchor %r", self.anchor)
-        handler = linkparse.LinkFinder(self.get_content(),
+    def get_anchors (self):
+        """Store list of anchors for this URL. Precondition: this URL is
+        an HTML resource."""
+        log.debug(LOG_CHECK, "Getting HTML anchors %s", self)
+        handler = linkparse.LinkFinder(self.get_content(), self.add_anchor,
                                    tags={'a': [u'name'], None: [u'id']})
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
-        parser.feed(self.get_content())
-        parser.flush()
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
         parser.handler = None
-        if any(x for x in handler.urls if x[0] == self.anchor):
+
+    def add_anchor (self, url, line, column, name, base):
+        """Add anchor URL."""
+        self.anchors.append((url, line, column, name, base))
+
+    def check_anchor (self):
+        """If URL was valid and has an anchor, check it. A warning is
+        logged if the anchor is not found.
+        """
+        if not self.aggregate.config["anchors"]:
             return
-        self.add_warning(_("Anchor #%(name)s not found.") %
-            {"name": self.anchor}, tag=WARN_URL_ANCHOR_NOT_FOUND)
+        log.debug(LOG_CHECK, "checking anchor %r", self.anchor)
+        if any(x for x in self.anchors if x[0] == self.anchor):
+            return
+        anchors = u",".join(u"`%s'" % x[0] for x in self.anchors)
+        args = {"name": self.anchor, "anchors": anchors}
+        msg = u"%s %s" % (_("Anchor `%(name)s' not found.") % args,
+                          _("Available anchors: %(anchors)s.") % args)
+        self.add_warning(msg, tag=WARN_URL_ANCHOR_NOT_FOUND)
 
     def set_extern (self, url):
         """
@@ -596,27 +619,33 @@ class UrlBase (object):
         return
 
     def can_get_content (self):
-        """
-        Indicate wether url get_content() can be called.
-        """
+        """Indicate wether url get_content() can be called."""
         return True
 
     def get_content (self):
-        """
-        Precondition: url_connection is an opened URL.
-        """
+        """Precondition: url_connection is an opened URL."""
         if self.data is None:
+            log.debug(LOG_CHECK, "Get content of %r", self.url)
             t = time.time()
-            self.data = self.url_connection.read()
+            self.data = self.read_content()
             self.dltime = time.time() - t
             self.dlsize = len(self.data)
         return self.data
 
+    def read_content (self):
+        """Return data for this URL. Can be overridden in subclasses."""
+        return self.url_connection.read()
+
     def check_content (self):
         """Check content data for warnings, syntax errors, viruses etc."""
-        if not (self.can_get_content() and self.valid):
-            # no data to check
+        if not (self.valid and self.can_get_content()):
             return
+        if self.is_html():
+            self.set_title_from_content()
+            if self.aggregate.config["anchors"]:
+                self.get_anchors()
+        if self.anchor:
+            self.check_anchor()
         warningregex = self.aggregate.config["warningregex"]
         if warningregex:
             log.debug(LOG_CHECK, "checking content")
@@ -808,31 +837,31 @@ class UrlBase (object):
         return None, None
 
     def parse_html (self):
-        """
-        Parse into HTML content and search for URLs to check.
+        """Parse into HTML content and search for URLs to check.
         Found URLs are added to the URL queue.
         """
         log.debug(LOG_CHECK, "Parsing HTML %s", self)
         # construct parser object
-        handler = linkparse.LinkFinder(self.get_content())
+        handler = linkparse.LinkFinder(self.get_content(), self.add_url)
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
-        parser.feed(self.get_content())
-        parser.flush()
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
         parser.handler = None
-        for url, line, column, name, codebase in handler.urls:
-            if codebase:
-                base_ref = codebase
-            else:
-                base_ref = handler.base_ref
-            base_ref = urlutil.url_norm(base_ref)[0]
-            url_data = get_url_from(url,
-                  self.recursion_level+1, self.aggregate, parent_url=self.url,
-                  base_ref=base_ref, line=line, column=column, name=name)
-            self.aggregate.urlqueue.put(url_data)
+
+    def add_url (self, url, line, column, name, base):
+        """Queue URL data for checking."""
+        base_ref = urlutil.url_norm(base)[0]
+        url_data = get_url_from(url, self.recursion_level+1, self.aggregate,
+            parent_url=self.url, base_ref=base_ref, line=line, column=column,
+            name=name)
+        self.aggregate.urlqueue.put(url_data)
 
     def parse_opera (self):
         """Parse an opera bookmark file."""
@@ -899,6 +928,37 @@ class UrlBase (object):
                          parent_url=self.url)
             self.aggregate.urlqueue.put(url_data)
 
+    def parse_word (self):
+        """Parse a word file for hyperlinks."""
+        if not winutil.has_word():
+            return
+        filename = self.get_temp_filename()
+        # open word file and parse hyperlinks
+        try:
+            app = winutil.get_word_app()
+            try:
+                doc = winutil.open_wordfile(app, filename)
+                try:
+                    for link in doc.Hyperlinks:
+                        url_data = get_url_from(link.Address,
+                                 self.recursion_level+1, self.aggregate,
+                                 parent_url=self.url, name=link.TextToDisplay)
+                        self.aggregate.urlqueue.put(url_data)
+                finally:
+                    winutil.close_wordfile(doc)
+            finally:
+                winutil.close_word_app(app)
+        except winutil.Error, msg:
+            log.warn(LOG_CHECK, "Error parsing word file: %s", msg)
+
+    def get_temp_filename (self):
+        """Get temporary filename for content to parse."""
+        # store content in temporary file
+        fd, filename = tempfile.mkstemp(suffix='.doc', prefix='lc_')
+        fp = os.fdopen(fd)
+        fp.write(self.get_content())
+        fp.close()
+
     def serialized (self):
         """
         Return serialized url check data as unicode string.
@@ -951,6 +1011,59 @@ class UrlBase (object):
         """
         return u"<%s >" % self.serialized()
 
+    def to_wire (self):
+        """Return a simplified transport object for logging.
+
+        The transport object must contain these attributes:
+        - url_data.valid: bool
+          Indicates if URL is valid
+        - url_data.cached: bool
+          Indicates if URL data has been loaded from cache.
+        - url_data.result: unicode
+          Result string
+        - url_data.warnings: list of unicode
+          List of tagged warnings for this URL.
+        - url_data.name: unicode string or None
+          name of URL (eg. filename or link name)
+        - url_data.parent_url: unicode or None
+          Parent URL
+        - url_data.base_ref: unicode or None
+          HTML base reference URL of parent
+        - url_data.url: unicode or None
+          Fully qualified URL.
+        - url_data.checktime: int
+          Number of seconds needed to check this link, default: zero.
+        - url_data.dltime: int
+          Number of seconds needed to download URL content, default: -1
+        - url_data.dlsize: int
+          Size of downloaded URL content, default: -1
+        - url_data.info: list of unicode
+          Additional information about this URL.
+        - url_data.line: int
+          Line number of this URL at parent document, or -1
+        - url_data.column: int
+          Column number of this URL at parent document, or -1
+        """
+        return containers.AttrDict(valid=self.valid,
+          extern=self.extern[0],
+          cached=self.cached,
+          result=self.result,
+          warnings=[x[1] for x in self.warnings],
+          name=self.name or u"",
+          title=self.get_title(),
+          parent_url=self.parent_url or u"",
+          base_ref=self.base_ref or u"",
+          base_url=self.base_url or u"",
+          url=self.url or u"",
+          checktime=self.checktime,
+          dltime=self.dltime,
+          dlsize=self.dlsize,
+          info=self.info,
+          line=self.line,
+          column=self.column,
+          cache_url_key=self.cache_url_key,
+        )
+
 
 def filter_tidy_errors (errors):
     """Filter certain errors from HTML tidy run."""
