@@ -4,6 +4,8 @@
 The user of the file doesn't have to worry about the compression,
 but random access is not allowed."""
 
+# Copied from Python source; License: Python License
+# Copyright Guido van Rossum <guido@cwi.nl> and others
 # based on Andrew Kuchling's minigzip.py distributed with the zlib module
 
 import struct, sys, time
@@ -460,6 +462,14 @@ class GzipFile:
         else:
             raise StopIteration
 
+    def __enter__(self):
+        if self.fileobj is None:
+            raise ValueError("I/O operation on closed GzipFile object")
+        return self
+
+    def __exit__(self, *args):
+        self.close()
+
 
 def _test():
     # Act like gzip; with -d, act like gunzip.
