@@ -1,4 +1,6 @@
 # -*- coding: iso-8859-1 -*-
+# Copied from Python source; License: Python License
+# Copyright Guido van Rossum <guido@cwi.nl> and others
 """HTTP/1.1 client library
 
 <intro stuff goes here>
@@ -357,7 +359,7 @@ class HTTPResponse:
         if not line:
             # Presumably, the server closed the connection before
             # sending a valid response.
-            raise BadStatusLine("Bad status line %r" % line)
+            raise BadStatusLine("Empty status line")
         try:
             [version, status, reason] = line.split(None, 2)
         except ValueError:
@@ -622,7 +624,7 @@ class HTTPResponse:
         while amt > 0:
             chunk = self.fp.read(min(amt, MAXAMOUNT))
             if not chunk:
-                raise IncompleteRead(s)
+                raise IncompleteRead(''.join(s), amt)
             s.append(chunk)
             amt -= len(chunk)
         return ''.join(s)
@@ -743,7 +745,7 @@ class HTTPConnection:
             else:
                 self.sock.sendall(str)
         except socket.error, v:
-            if v[0] == 32:      # Broken pipe
+            if v.args[0] == 32:      # Broken pipe
                 self.close()
             raise
 
@@ -916,7 +918,7 @@ class HTTPConnection:
             self._send_request(method, url, body, headers)
         except socket.error, v:
             # trap 'Broken pipe' if we're allowed to automatically reconnect
-            if v[0] != errno.EPIPE or not self.auto_open:
+            if v.args[0] != errno.EPIPE or not self.auto_open:
                 raise
             # try one more time
             self._send_request(method, url, body, headers)
@@ -926,7 +928,7 @@ class HTTPConnection:
         thelen = None
         try:
             thelen = str(len(body))
-        except TypeError, te:
+        except TypeError:
             # If this is a file-like object, try to
             # fstat its file descriptor
             import os
@@ -1071,17 +1073,17 @@ class SSLFile(SharedSocketClient):
             try:
                 buf = self._ssl.read(self._bufsize)
             except socket.sslerror, err:
-                if (err[0] == socket.SSL_ERROR_WANT_READ
-                    or err[0] == socket.SSL_ERROR_WANT_WRITE):
+                if (err.args[0] == socket.SSL_ERROR_WANT_READ
+                    or err.args[0] == socket.SSL_ERROR_WANT_WRITE):
                     continue
-                if (err[0] == socket.SSL_ERROR_ZERO_RETURN
-                    or err[0] == socket.SSL_ERROR_EOF):
+                if (err.args[0] == socket.SSL_ERROR_ZERO_RETURN
+                    or err.args[0] == socket.SSL_ERROR_EOF):
                     break
                 raise
             except socket.error, err:
-                if err[0] == errno.EINTR:
+                if err.args[0] == errno.EINTR:
                     continue
-                if err[0] == errno.EBADF:
+                if err.args[0] == errno.EBADF:
                     # XXX socket was closed?
                     break
                 raise
@@ -1342,9 +1344,18 @@ class UnimplementedFileMode(HTTPException):
     pass
 
 class IncompleteRead(HTTPException):
-    def __init__(self, partial):
+    def __init__(self, partial, expected=None):
         self.args = partial,
         self.partial = partial
+        self.expected = expected
+    def __repr__(self):
+        if self.expected is not None:
+            e = ', %i more expected' % self.expected
+        else:
+            e = ''
+        return 'IncompleteRead(%i bytes read%s)' % (len(self.partial), e)
+    def __str__(self):
+        return repr(self)
 
 class ImproperConnectionState(HTTPException):
     pass
