@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2009 Bastian Kleineidam
+# Copyright (C) 2004-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,7 @@
 """
 Test news checking.
 """
-from tests import has_network
-from nose import SkipTest
+from tests import need_network
 from . import LinkCheckTest
 
 
@@ -27,9 +26,8 @@ class TestHttps (LinkCheckTest):
     Test https: link checking.
     """
 
+    @need_network
     def test_https (self):
-        if not has_network():
-            raise SkipTest("no network available")
         url = u"https://www.amazon.de/"
         resultlines = [
             u"url %s" % url,
