@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006-2009 Bastian Kleineidam
+# Copyright (C) 2006-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,29 +18,26 @@
 Test virus filter.
 """
 import unittest
-from tests import has_clamav
-from nose import SkipTest
+from tests import need_clamav
 from linkcheck import clamav
 
 
 class TestClamav (unittest.TestCase):
 
+    @need_clamav
     def testClean (self):
-        if not has_clamav():
-            raise SkipTest("no ClamAV available")
         data = ""
         infected, errors = clamav.scan(data)
         self.assertFalse(infected)
         self.assertFalse(errors)
 
+    @need_clamav
     def testInfected (self):
-        if not has_clamav():
-            raise SkipTest("no ClamAV available")
         data = '<object data="&#109;s-its:mhtml:file://'+ \
                'C:\\foo.mht!${PATH}/' + \
                'EXPLOIT.CHM::' + \
                '/exploit.htm">'
         infected, errors = clamav.scan(data)
         msg = 'stream: Exploit.HTML.MHTRedir.2n FOUND\n'
-        self.assert_(msg in infected)
+        self.assertTrue(msg in infected)
         self.assertFalse(errors)
