@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2008-2009 Bastian Kleineidam
+# Copyright (C) 2008-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,21 +19,18 @@ Test network functions.
 """
 
 import unittest
-from tests import has_posix
-from nose import SkipTest
+from tests import need_posix
 import linkcheck.network
 
 
 class TestNetwork (unittest.TestCase):
     """Test network functions."""
 
+    @need_posix
     def test_ifreq_size (self):
-        if not has_posix():
-            raise SkipTest("no POSIX system")
         self.assertTrue(linkcheck.network.ifreq_size() > 0)
 
+    @need_posix
     def test_interfaces (self):
-        if not has_posix():
-            raise SkipTest("no POSIX system")
         ifc = linkcheck.network.IfConfig()
         ifc.getInterfaceList()
